#include "MathM.h"

//   PI     1 - (1/3) + (1/5) - (1/7) + ... = PI/4
double PI(double max_d)
{
    //            
    //             
	static double rez;
	static double d;
    
    rez = 1;
    d = 3;

	do {
		rez -= (1/d);
		d +=2;
		rez += (1/d);
		d +=2;
	} while (d < max_d);

	return rez;
}

DWORD GetFloatTime1(bool *pRez)
{
	double p;
	DWORD time_start;

	time_start = GetTickCount();
	p = 4*PI(60000000);

	*pRez = true;
	return (GetTickCount() - time_start);
}

//   - Sin(X)  0 <= X <= PI   
DWORD GetFloatTime2(bool *pRez)
{
	DWORD time_start;
	static double knot;
	static double step;
	static double result;

    step = 0.0000004;
    result = 0;

	time_start = GetTickCount();

	for (knot = 0; knot <= 3.1415926535897932; knot += step) {
		result += (sin(knot)*step);
	}
	*pRez = true;
	return (GetTickCount() - time_start);
}

#define MATR(X, Y) pMatr[(X)*S + (Y)]

//     SxS   N.
// (.         =  )
bool matrix(DWORD S, DWORD N)
{
	DWORD idx1, idx2, idx3;
	DWORD cnt;
	DWORD* pMatr;
	DWORD r;

	if ((S < 1) || (N < 2)) return false; //   .
	N--;

	pMatr = (DWORD*)LocalAlloc(LMEM_FIXED|LMEM_ZEROINIT, S*S*sizeof(DWORD));

	if (pMatr) {
		for (idx1 = 0; idx1 < S; idx1++) {
			MATR(idx1,idx1) = 1; //      .
		}
		
		for (cnt = 0; cnt < N; cnt++) {
			for (idx2 = 0; idx2 < S; idx2++) {
				for (idx1 = 0; idx1 < S; idx1++) {
					// MATR(idx1,idx2) = MATR(idx1,1)*MATR(1,idx2) + MATR(idx1,2)*MATR(2,idx2) + ...
					r = 0;
					for (idx3 = 0; idx3 < S; idx3++) {
						 r += MATR(idx1,idx3)*MATR(idx3,idx2);
					}
					MATR(idx1,idx2) = r;
				}
			}
		}
		LocalFree(pMatr);
		return true;
	}
	return false; //      .
}

DWORD GetMatrixTime(bool *pRez, DWORD S, DWORD N)
{
	DWORD time_start = GetTickCount();
	*pRez = matrix(S, N);
	return (GetTickCount() - time_start);
}

DWORD GetMatrixTime1(bool *pRez)
{
	return GetMatrixTime(pRez, 700, 2); //  ~1.87  . 
}

DWORD GetMatrixTime2(bool *pRez)
{
	return GetMatrixTime(pRez, 70, 1000); //  ~20 .    d-cache
}

DWORD GetMatrixTime3(bool *pRez)
{
	return GetMatrixTime(pRez, 10, 100000); //  400 .    Mini-data cache
}

//   .
DWORD Rnd(DWORD x) 
{
	return (31*x + 112) % 0xFFFF; 
}

//    .
void SortMass(DWORD *pMas, DWORD masSize)
{
	DWORD idx1, idx2;
	DWORD minVal, minIdx;

	for (idx1 = 0; idx1 < masSize; idx1++) {
		minVal = pMas[idx1];
		minIdx = idx1;
		for (idx2 = idx1; idx2 < masSize; idx2++) {
			if (pMas[idx2] < minVal) {
				minVal = pMas[idx2];
				minIdx = idx2;
			}
		}
		pMas[minIdx] = pMas[idx1];
		pMas[idx1] = minVal;		
	}
}

DWORD GetSortTime(bool *pRez, DWORD masSize, DWORD count)
{
	DWORD time_start = 0;
	DWORD time_finish = 0;
	DWORD rnd;
	DWORD *pMas = 0;
	DWORD cnt;
	DWORD idx;

	pMas = (DWORD*)LocalAlloc(LMEM_FIXED|LMEM_ZEROINIT, masSize*sizeof(DWORD));
	if (!pMas) {
		*pRez = false;
		return 0;
	}

	time_start = GetTickCount();
	for (cnt = 0; cnt < count; cnt++) {
		rnd = 0;
		for (idx = 0; idx < masSize; idx++) { //    
			rnd = Rnd(rnd);
			pMas[idx] = rnd;
		}
		SortMass(pMas, masSize);
	}
	time_finish = GetTickCount();
	LocalFree(pMas);
	*pRez = true;
	return (time_finish - time_start);
}

DWORD GetSortTime1(bool *pRez)
{
	return GetSortTime(pRez, 50000, 1); //  ~200 .
}

DWORD GetSortTime2(bool *pRez)
{
	return GetSortTime(pRez, 5000, 50); //  ~20 .
}

DWORD GetSortTime3(bool *pRez)
{
	return GetSortTime(pRez, 100, 50000); //  ~0.5 .
}

//      <= maximum.
DWORD PrimeSum(DWORD maximum)
{
	DWORD idx;
	DWORD number;
	DWORD sum = 0;

	for (number = 2; number <= maximum; number++) {
		for (idx = 2; idx < number; idx++) {
			if (number == (number/idx)*idx) break;
		}
		if (idx == number) {
			sum += number;
		}
	}
	return sum;
}

DWORD GetPrimeTime(bool *pRez)
{
	DWORD time_start;
	DWORD sum;

	time_start = GetTickCount();
	sum = PrimeSum(50000);

	*pRez = true;
	return (GetTickCount() - time_start);
}