#ifndef _UNICODE
  #define _UNICODE
#endif

#ifndef UNICODE
  #define UNICODE
#endif

#ifdef UNDER_CE
  #define CMD_LINE_TYPE LPWSTR
  #define WIN_STYLE WS_VISIBLE
#else
  #define CMD_LINE_TYPE LPSTR
  #define WIN_STYLE (WS_VISIBLE | WS_OVERLAPPEDWINDOW)
#endif

#include <windows.h>
#include <stdio.h>
#include "MathM.h"

BOOL InitInstance(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

HINSTANCE g_hInst;
const WCHAR *g_tszWindowClass = L"MathTest";
LPCTSTR g_tszTitle = L"MathTest";

HWND g_hFloatButton1;
HWND g_hFloatText1;
HWND g_hFloatButton2;
HWND g_hFloatText2;
HWND g_hMatrixButton1;
HWND g_hMatrixText1;
HWND g_hMatrixButton2;
HWND g_hMatrixText2;
HWND g_hMatrixButton3;
HWND g_hMatrixText3;
HWND g_hSortButton1;
HWND g_hSortText1;
HWND g_hSortButton2;
HWND g_hSortText2;
HWND g_hSortButton3;
HWND g_hSortText3;
HWND g_hPrimeButton;
HWND g_hPrimeText;
HWND g_hStartAllButton;
HWND g_hStartAllText;
HWND g_hExitButton;

#define FLOAT_BUTTON1 0
#define FLOAT_BUTTON2 1
#define MATRIX_BUTTON1 2
#define MATRIX_BUTTON2 3
#define MATRIX_BUTTON3 4
#define SORT_BUTTON1 5
#define SORT_BUTTON2 6
#define SORT_BUTTON3 7
#define PRIME_BUTTON 8
#define START_ALL_BUTTON 9
#define EXIT_BUTTON 10

//===========================================================================================
#define AddToLog(x) LogEvent x
void LogEvent(const char *Format, ...)
{
    va_list ArgList;
    char szOutString[100];
    FILE *pLogFile;
    
    va_start(ArgList, Format);    
    _vsnprintf(szOutString, sizeof(szOutString), Format, ArgList);

    pLogFile = fopen("MathTest.log", "a");

    if (pLogFile) { 
        fprintf(pLogFile, "%s", &szOutString[0]);
        fclose(pLogFile);
    }
}
//===========================================================================================
void InitInfo()
{
    SYSTEMTIME  stTime;
    SYSTEM_INFO sysInfo;

    GetLocalTime(&stTime);
    GetSystemInfo(&sysInfo);
    AddToLog(("Started. %02d.%02d.%04d %02d:%02d:%02d\n", stTime.wDay, stTime.wMonth, 
                    stTime.wYear, stTime.wHour, stTime.wMinute, stTime.wSecond));
    AddToLog(("Processor. Arch:%x Level:%x Revision:%x\n", sysInfo.wProcessorArchitecture,
                    sysInfo.wProcessorLevel, sysInfo.wProcessorRevision));
}
//===========================================================================================
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, CMD_LINE_TYPE lpCmdLine, int nCmdShow)
{
    MSG msg;
    HANDLE hMutex = NULL;
	
    hMutex = CreateMutex(0, 0, L"_CE_MATH_EXE_MUTEX_");	
    if (0 != hMutex) {
        if (ERROR_ALREADY_EXISTS == GetLastError()) {
            HWND hwnd;
            CloseHandle(hMutex);
            hMutex = 0;            
            hwnd = FindWindow(g_tszWindowClass, g_tszTitle);
            if (0 == hwnd) {
                Sleep(500);
                hwnd = FindWindow(g_tszWindowClass, g_tszTitle);
            }            
            if (0 != hwnd) {
                SetForegroundWindow((HWND) (((ULONG) hwnd) | 0x01));                
                return FALSE;
            }            
        }        
    } else return FALSE;   
	
    if (!InitInstance (hInstance, nCmdShow)) {
        return FALSE;
    }
    InitInfo();

    while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
    }    
	if (0 != hMutex) {
		CloseHandle(hMutex);
		hMutex = 0;
	}
    return msg.wParam;
}
//===========================================================================================
#define TOP_BUT(X)	(4 + 26*(X))
#define LEFT_BUT(X) 6
#define LEN_BUT(X)	116
#define HEG_BUT(X)	24

#define TOP_TXT(X)	(6 + 26*(X))
#define LEFT_TXT(X)	124
#define LEN_TXT(X)	110
#define HEG_TXT(X)	24

HWND CreateButton(HWND parent, DWORD num, WCHAR *txt)
{
	return CreateWindow(L"button", txt, WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON,
		LEFT_BUT(num), TOP_BUT(num), LEN_BUT(num), HEG_BUT(num), 
		parent, (HMENU)num, g_hInst, 0);
}

HWND CreateLabel(HWND parent, DWORD num)
{
	return CreateWindow(L"static", L"no data", WS_CHILD | WS_VISIBLE | SS_CENTER,
		LEFT_TXT(num), TOP_TXT(num), LEN_TXT(num), HEG_TXT(num), parent, (HMENU)-1, g_hInst, 0);
}
//===========================================================================================
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    HWND    hWnd;
    WNDCLASS wc;
    
    g_hInst = hInstance;	
    // Register the class
    wc.style            = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc      = (WNDPROC) WndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    wc.hInstance        = g_hInst;
    wc.hIcon            = 0; //LoadIcon(0, IDI_APPLICATION);
    wc.hCursor          = LoadCursor(0, IDC_ARROW);
    wc.hbrBackground    = (HBRUSH)(COLOR_WINDOW);
    wc.lpszMenuName     = 0;
    wc.lpszClassName    = g_tszWindowClass;
    
    if (!RegisterClass(&wc)) return FALSE;
	
    hWnd = CreateWindow(g_tszWindowClass, g_tszTitle, WIN_STYLE, 
		CW_USEDEFAULT, CW_USEDEFAULT, 240, 320, 0, 0, g_hInst, 0);
    
    if (!hWnd) return FALSE;
    
    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd); 

	g_hFloatButton1 = CreateButton(hWnd, FLOAT_BUTTON1, L"Calc PI"); 
	g_hFloatText1 = CreateLabel(hWnd, FLOAT_BUTTON1);

	g_hFloatButton2 = CreateButton(hWnd, FLOAT_BUTTON2, L"Integration"); 
	g_hFloatText2 = CreateLabel(hWnd, FLOAT_BUTTON2);

	g_hMatrixButton1 = CreateButton(hWnd, MATRIX_BUTTON1, L"Matrix Big"); 
	g_hMatrixText1 = CreateLabel(hWnd, MATRIX_BUTTON1);

	g_hMatrixButton2 = CreateButton(hWnd, MATRIX_BUTTON2, L"Matrix Med."); 
	g_hMatrixText2 = CreateLabel(hWnd, MATRIX_BUTTON2);

	g_hMatrixButton3 = CreateButton(hWnd, MATRIX_BUTTON3, L"Matrix Small"); 
	g_hMatrixText3 = CreateLabel(hWnd, MATRIX_BUTTON3);

	g_hSortButton1 = CreateButton(hWnd, SORT_BUTTON1, L"Sort Big"); 
	g_hSortText1 = CreateLabel(hWnd, SORT_BUTTON1);

	g_hSortButton2 = CreateButton(hWnd, SORT_BUTTON2, L"Sort Med."); 
	g_hSortText2 = CreateLabel(hWnd, SORT_BUTTON2);

	g_hSortButton3 = CreateButton(hWnd, SORT_BUTTON3, L"Sort Small"); 
	g_hSortText3 = CreateLabel(hWnd, SORT_BUTTON3);

	g_hPrimeButton = CreateButton(hWnd, PRIME_BUTTON, L"Prime num."); 
	g_hPrimeText = CreateLabel(hWnd, PRIME_BUTTON);

    g_hStartAllButton = CreateButton(hWnd, START_ALL_BUTTON, L"Start all tests"); 
    g_hStartAllText = CreateLabel(hWnd, START_ALL_BUTTON);

	g_hExitButton = CreateButton(hWnd, EXIT_BUTTON, L"Exit"); 

    return TRUE;
}
//===========================================================================================
VOID EnableControls(BOOL b)
{
	EnableWindow(g_hFloatButton1, b);
	EnableWindow(g_hFloatButton2, b);
	EnableWindow(g_hMatrixButton1, b);
	EnableWindow(g_hMatrixButton2, b);
	EnableWindow(g_hMatrixButton3, b);
	EnableWindow(g_hSortButton1, b);
	EnableWindow(g_hSortButton2, b);
	EnableWindow(g_hSortButton3, b);
	EnableWindow(g_hPrimeButton, b);
	EnableWindow(g_hStartAllButton, b);
    EnableWindow(g_hExitButton, b);
}
//===========================================================================================
DWORD RunTest(HWND button, HWND label, TestFunc func)
{
	DWORD duration;
	WCHAR buf[20];
	bool rez;

	EnableControls(FALSE);
	SetWindowText(label, L"please wait!");
	duration = func(&rez);

    if (0 != GetWindowText(button, buf, sizeof(buf)/sizeof(WCHAR))) {
        AddToLog(("%S = ", buf));
    }
	if (rez) {
		swprintf(buf, L"%.02f secs", (double(duration/10))/100);
	} else {
		swprintf(buf, L"error!");
	}

	SetWindowText(label, buf);
    AddToLog(("%S\n", buf));
	EnableControls(TRUE);

    return duration;
}
//===========================================================================================
void StartAllTests()
{
    DWORD sum;
    WCHAR buf[24];

    sum = RunTest(g_hFloatButton1, g_hFloatText1, GetFloatTime1);
    sum += RunTest(g_hFloatButton2, g_hFloatText2, GetFloatTime2);
    sum += RunTest(g_hMatrixButton1, g_hMatrixText1, GetMatrixTime1);
    sum += RunTest(g_hMatrixButton2, g_hMatrixText2, GetMatrixTime2);
    sum += RunTest(g_hMatrixButton3, g_hMatrixText3, GetMatrixTime3);
    sum += RunTest(g_hSortButton1, g_hSortText1, GetSortTime1);
    sum += RunTest(g_hSortButton2, g_hSortText2, GetSortTime2);
    sum += RunTest(g_hSortButton3, g_hSortText3, GetSortTime3);
    sum += RunTest(g_hPrimeButton, g_hPrimeText, GetPrimeTime);
    swprintf(buf, L"sum=%.02f", (double(sum/10))/100);
    SetWindowText(g_hStartAllText, buf);
    AddToLog(("%S\n", buf));
}
//===========================================================================================
LRESULT CALLBACK WndProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
    switch (Msg) {        
	case WM_CREATE:
		break;
		
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
		
	case WM_COMMAND:
		if (wParam == FLOAT_BUTTON1) RunTest(g_hFloatButton1, g_hFloatText1, GetFloatTime1);
		if (wParam == FLOAT_BUTTON2) RunTest(g_hFloatButton2, g_hFloatText2, GetFloatTime2);
		if (wParam == MATRIX_BUTTON1) RunTest(g_hMatrixButton1, g_hMatrixText1, GetMatrixTime1);
		if (wParam == MATRIX_BUTTON2) RunTest(g_hMatrixButton2, g_hMatrixText2, GetMatrixTime2);
		if (wParam == MATRIX_BUTTON3) RunTest(g_hMatrixButton3, g_hMatrixText3, GetMatrixTime3);
		if (wParam == SORT_BUTTON1) RunTest(g_hSortButton1, g_hSortText1, GetSortTime1);
		if (wParam == SORT_BUTTON2) RunTest(g_hSortButton2, g_hSortText2, GetSortTime2);
		if (wParam == SORT_BUTTON3) RunTest(g_hSortButton3, g_hSortText3, GetSortTime3);
		if (wParam == PRIME_BUTTON) RunTest(g_hPrimeButton, g_hPrimeText, GetPrimeTime);
		if (wParam == START_ALL_BUTTON) StartAllTests();
		if (wParam == EXIT_BUTTON) SendMessage (hWnd, WM_CLOSE, 0, 0);
		break;
						
    default:
        return DefWindowProc(hWnd, Msg, wParam, lParam);
        break;
    }	
    return 0;
}
//===========================================================================================